package com.telpo.dipperposition;

import com.telpo.dipperposition.server.DipperPositionServer;
import com.telpo.dipperposition.server.TimeAsyncServer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.context.properties.ConfigurationPropertiesScan;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.core.env.ConfigurableEnvironment;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;

/**
 * @program: gateway
 * @description: 网关启动类
 * @author: linwl
 * @create: 2020-06-18 16:39
 */
@SpringBootApplication
@EnableDiscoveryClient
@ConfigurationPropertiesScan
@EnableAsync
@EnableScheduling
@Slf4j
public class DipperPositionApplication {

  public static void main(String[] args) {
    try {
      log.info("北斗定位服务开始!");
      ConfigurableApplicationContext applicationContext = SpringApplication.run(DipperPositionApplication.class, args);
      log.info("北斗定位服务启动!");
      //启动服务端
      ConfigurableEnvironment environment = applicationContext.getEnvironment();

      DipperPositionServer nettyServer = new DipperPositionServer(environment);
      nettyServer.start();
    } catch (Exception e) {
      log.error(e.getMessage());
    }
  }
}
