package com.telpo.dipperposition.server;

import com.alibaba.nacos.api.exception.NacosException;
import com.telpo.dipperposition.handler.ServerChannelInitializer;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.env.ConfigurableEnvironment;

import java.net.InetSocketAddress;

/**
 * @program: TimeAsyncServer
 * @description: 北斗定位
 * @author: king
 * @create: 2021-01-13 14:01
 */
@Slf4j
public class TimeAsyncServer {

    // @Autowired PositionConfigInfo positionConfigInfo;
    private String serverAddr;
    private Integer serverPort;
    private Integer timeAsycPort;
    private Integer posAsycPort;
    private Integer starsAsycPort;

    public TimeAsyncServer(ConfigurableEnvironment environment) throws NacosException {

        this.serverAddr = environment.getProperty("position-server.serverAddr");
        this.serverPort = Integer.parseInt(environment.getProperty("server.port"));
        this.timeAsycPort = Integer.parseInt(environment.getProperty("position-server.timeAsycPort"));
        this.posAsycPort = Integer.parseInt(environment.getProperty("position-server.posAsycPort"));
        this.starsAsycPort = Integer.parseInt(environment.getProperty("position-server.starsAsycPort"));
    }

    public void start() {

        //new 一个主线程组
        EventLoopGroup mainThreadGroup = new NioEventLoopGroup(1);
        //new 一个工作线程组
        EventLoopGroup workThreadGroup = new NioEventLoopGroup(200);
        //InetSocketAddress socketAddress = new InetSocketAddress(serverAddr,null);
        ServerBootstrap bootstrap = new ServerBootstrap()
                .group(mainThreadGroup, workThreadGroup)
                .channel(NioServerSocketChannel.class)
                .childHandler(new ServerChannelInitializer())
                //.localAddress(socketAddress)
                //设置队列大小
                .option(ChannelOption.SO_BACKLOG, 1024)
                // 两小时内没有数据的通信时,TCP会自动发送一个活动探测数据报文
                .childOption(ChannelOption.SO_KEEPALIVE, true);


        //绑定端口,开始接收进来的连接
        try {
            ChannelFuture channelFuture = bootstrap.bind(timeAsycPort).sync();
            log.info("时间辅助服务器启动开始监听端口: {}", timeAsycPort);
            channelFuture.addListener(future -> {
                if (future.isSuccess()){
                    System.out.println("start success");
                }else{
                    System.out.println("start failed");
                }
            });

            channelFuture.channel().closeFuture().sync();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            //关闭主线程组
            mainThreadGroup.shutdownGracefully();
            //关闭工作线程组
            workThreadGroup.shutdownGracefully();
        }
    }
}
